 /*
 * file      : build.c
 * project   : BaseBuild
 * with      : Gtk-2
 *
 * copyright : (C) 2010 by Claude Bulin
 *
 * BaseBuild - Creation d'une base de programmation en langage C de type GNU avec les autotools
 * GNU General Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * OLD ADRESS:
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * NEW ADRESS:
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gstdio.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <pthread.h>

#include "../libbasebuild/libbasebuild.h"
#include "global.h"
#include "configuser.h"
#include "callbacks.h"
#include "build.h"
#include "view.h"
#include "win_info.h"
#include "def_src.h"
#include "proto.h"



extern VAR_CALLBACKS VarCallbacks;
VAR_BUILD VarBuild;



// 
// 
gchar *build_get_name_project (void)
{
	return ((gchar *)gtk_entry_get_text (GTK_ENTRY (VarCallbacks.Adr_name_project)));
}

// --------------------------------------------------------------------------
// 
// GENERATION EN MEMOIRE
// 
// --------------------------------------------------------------------------

// FREE ALLOCATIONS
// 
void build_remove (void)
{
	if (VarBuild.GstrMakefileAm != NULL) {
		g_string_free (VarBuild.GstrMakefileAm, TRUE);
		VarBuild.GstrMakefileAm = NULL;
	}
	if (VarBuild.GstrConfigureIn != NULL) {
		g_string_free (VarBuild.GstrConfigureIn, TRUE);
		VarBuild.GstrConfigureIn = NULL;
	}
	if (VarBuild.GstrLibMakefileAm != NULL) {
		g_string_free (VarBuild.GstrLibMakefileAm, TRUE);
		VarBuild.GstrLibMakefileAm = NULL;
	}
	if (VarBuild.GstrPixmapsMakefileAm != NULL) {
		g_string_free (VarBuild.GstrPixmapsMakefileAm, TRUE);
		VarBuild.GstrPixmapsMakefileAm = NULL;
	}
	if (VarBuild.GstrMenuMakefileAm != NULL) {
		g_string_free (VarBuild.GstrMenuMakefileAm, TRUE);
		VarBuild.GstrMenuMakefileAm = NULL;
	}
	if (VarBuild.GstrMimeMakefileAm != NULL) {
		g_string_free (VarBuild.GstrMimeMakefileAm, TRUE);
		VarBuild.GstrMimeMakefileAm = NULL;
	}
	if (VarBuild.GstrGladeMakefileAm != NULL) {
		g_string_free (VarBuild.GstrGladeMakefileAm, TRUE);
		VarBuild.GstrGladeMakefileAm = NULL;
	}
	
	VarBuild.ListPathSrc = DefSrc_remove_list (VarBuild.ListPathSrc);
	
	if (NULL != VarBuild.PathProject) {
		g_free (VarBuild.PathProject);
		VarBuild.PathProject = NULL;
	}
	if (NULL != VarBuild.NameProject) {
		g_free (VarBuild.NameProject);
		VarBuild.NameProject = NULL;
	}
	if (NULL != VarBuild.NameLib) {
		g_free (VarBuild.NameLib);
		VarBuild.NameLib = NULL;
	}
	if (NULL != VarBuild.VersionProject) {
		g_free (VarBuild.VersionProject);
		VarBuild.VersionProject = NULL;
	}
	if (NULL != VarBuild.NewPathName) {
		g_free (VarBuild.NewPathName);
		VarBuild.NewPathName = NULL;
	}
	if (NULL != VarBuild.PathNameLib) {
		g_free (VarBuild.PathNameLib);
		VarBuild.PathNameLib = NULL;
	}
	if (NULL != VarBuild.PathNameMan) {
		g_free (VarBuild.PathNameMan);
		VarBuild.PathNameMan = NULL;
	}
	if (NULL != VarBuild.PathNamePo) {
		g_free (VarBuild.PathNamePo);
		VarBuild.PathNamePo = NULL;
	}
	if (NULL != VarBuild.PathNamePixmaps) {
		g_free (VarBuild.PathNamePixmaps);
		VarBuild.PathNamePixmaps = NULL;
	}
	if (NULL != VarBuild.PathNameMenu) {
		g_free (VarBuild.PathNameMenu);
		VarBuild.PathNameMenu = NULL;
	}
	if (NULL != VarBuild.PathNameMime) {
		g_free (VarBuild.PathNameMime);
		VarBuild.PathNameMime = NULL;
	}
	if (NULL != VarBuild.PathNameGlade) {
		g_free (VarBuild.PathNameGlade);
		VarBuild.PathNameGlade = NULL;
	}
}
// ALLOCATIONS OPTIONS
// 
void build_maj_options (void)
{
	VAR_BUILD_SRC	*DefSrc = NULL;
	GList		*List = NULL;
	gchar		*PtrLib = NULL;
	
	// FREE ALLOCATIONS
	build_remove ();
	
	VarBuild.ListPathSrc     = DefSrc_get_list ();
	VarBuild.PathProject     = g_strdup (gtk_button_get_label (GTK_BUTTON (VarCallbacks.Adr_button_path)));
	VarBuild.NameProject     = g_strdup (gtk_entry_get_text (GTK_ENTRY (VarCallbacks.Adr_name_project)));
	VarBuild.VersionProject  = g_strdup_printf ("%d.%d.%d",
					gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_version_A)),
					gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_version_B)),
					gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_version_C))
					);
	VarBuild.NewPathName     = g_strdup_printf ("%s/%s-%s", VarBuild.PathProject, VarBuild.NameProject, VarBuild.VersionProject);
	
	PtrLib = (gchar *)gtk_entry_get_text (GTK_ENTRY (VarCallbacks.Adr_name_library));
	if ('\0' == *PtrLib) {
		VarBuild.NameLib = g_strdup ("lib");
	}
	else {
		VarBuild.NameLib = g_strdup (PtrLib);
	}
	VarBuild.PathNameLib = g_strdup_printf ("%s/%s", VarBuild.NewPathName, VarBuild.NameLib);
	
	List = g_list_first (VarBuild.ListPathSrc);
	while (List) {
		if (NULL != (DefSrc = (VAR_BUILD_SRC *)List->data)) {
			DefSrc->PathName          = g_strdup_printf ("%s/%s", VarBuild.NewPathName, DefSrc->Name);
		}
		List = g_list_next (List);
	}
	
	VarBuild.PathNameMan     = g_strdup_printf ("%s/man", VarBuild.NewPathName);
	VarBuild.PathNamePo      = g_strdup_printf ("%s/po", VarBuild.NewPathName);
	VarBuild.PathNamePixmaps = g_strdup_printf ("%s/pixmaps", VarBuild.NewPathName);
	VarBuild.PathNameMenu    = g_strdup_printf ("%s/menu", VarBuild.NewPathName);
	VarBuild.PathNameMime    = g_strdup_printf ("%s/mime", VarBuild.NewPathName);
	VarBuild.PathNameGlade   = g_strdup_printf ("%s/glade", VarBuild.NewPathName);
	// gtk+-2.0
	// glib-2.0
	// glade/
	VarBuild.Bool_dossier_glade		= 
	VarBuild.Bool_lib_gtk			= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_lib_gtk")));
	// gthread-2.0
	// pthread
	VarBuild.Bool_lib_gthread		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_lib_gthread")));
	VarBuild.Bool_dossier_po		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_po")));
	VarBuild.Bool_dossier_manpage		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_manpage")));
	VarBuild.Bool_dossier_lib_statique	= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_lib_statique")));
	VarBuild.Bool_dossier_pixmaps		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_pixmaps")));
	VarBuild.Bool_dossier_menu		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_menu")));
	VarBuild.Bool_dossier_mime		= gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_dossier_mime")));
	VarBuild.Bool_import_glade_file         = FALSE;
	VarBuild.Bool_import_glade_translate    = FALSE;
	
	if (TRUE == VarBuild.Bool_lib_gtk &&
		TRUE == gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON (GLADE_GET_OBJECT("checkbutton_choice_glade_file")))) {
			
		gchar		*PathnameFileGlade = NULL;

		PathnameFileGlade = Callbacks_get_pathnamefile_glade ();
			
		if (TRUE == (VarBuild.Bool_import_glade_file = Proto_init (PathnameFileGlade))) {
			ProtoTranslate_init (PathnameFileGlade);
			VarBuild.Bool_import_glade_translate = Proto_get_buf_translate () != NULL ? TRUE : FALSE;
		}
			
		g_free (PathnameFileGlade);
		PathnameFileGlade = NULL;
	}
	g_print ("VarBuild.Bool_import_glade_file = %s\n", VarBuild.Bool_import_glade_file ? "TRUE" : "FALSE");
}

// --------------------------------------------------------------------------
// 
// GENERATION SUR MEMOIRE DE MASSE
// 
// --------------------------------------------------------------------------

// GENERATION 'Path + Name' avec le contenu de 'buffer'
// 
void build_create_file (gchar *Path, gchar *Name, gchar *Ext, gchar *buffer)
{
	gchar	*LineCommand = NULL;
	FILE	*fp = NULL;

	LineCommand = g_strdup_printf ("%s/%s%s", Path, Name, Ext);
	g_print("LineCommand = %s\n", LineCommand);
	
	if (NULL != (fp = fopen (LineCommand, "w"))) {
		if (NULL != buffer) {
			// g_fprintf (fp, buffer);
			fprintf (fp, "%s", buffer);
		}
		fclose (fp);
	}
	if (NULL != LineCommand) {
		g_free (LineCommand);
		LineCommand = NULL;
	}
}
// CREATION FICHIER lib/lib.c lib/lib.h
// 
void build_create_LIB_C_file (gchar *Path)
{
	GString	*Gstr = NULL;
	
	// CREATE lib.c
	// 
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, "/*\n");	
	g_string_append_printf (Gstr, " *  %s/lib.c\n", VarBuild.NameLib);	
	g_string_append_printf (Gstr, " */\n");	
	g_string_append_printf (Gstr, "\n");	
	g_string_append_printf (Gstr, "#include <stdio.h>\n");	
	g_string_append_printf (Gstr, "\n");	
	g_string_append_printf (Gstr, "void libH_print_mess (char *p_mess)\n");	
	g_string_append_printf (Gstr, "{\n");	
	g_string_append_printf (Gstr, "\tprintf(\"%%s\", p_mess);\n");	
	g_string_append_printf (Gstr, "}\n");	
	g_string_append_printf (Gstr, "\n");	
	build_create_file (Path, "lib", ".c", Gstr->str);	
	g_string_free (Gstr, TRUE);
	
	// CREATE lib.h
	// 
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, "/*\n");
	g_string_append_printf (Gstr, " *  %s/lib.h\n", VarBuild.NameLib);	
	g_string_append_printf (Gstr, " */\n");
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, "#ifndef __libH_h__\n");
	g_string_append_printf (Gstr, "#define __libH_h__ 1\n");
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, "void libH_print_mess (char *p_mess);\n");
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, "#endif\n");
	g_string_append_printf (Gstr, "\n");
	build_create_file (Path, "lib", ".h", Gstr->str);	
	g_string_free (Gstr, TRUE);
}
// CREATION FICHIER src/translate.c
// 
void build_create_SRC_TRANSLATE_C_file (VAR_BUILD_SRC *VarBuildSrc)
{
	// PRINT_FUNC_LF();

	if (TRUE == VarBuild.Bool_import_glade_file && TRUE == VarBuild.Bool_import_glade_translate) {
			
		GString	*GstrTranslate = NULL;
			
		if (NULL != (GstrTranslate = Proto_get_buf_translate ())) {
			view_set_text_va ("\t", VarBuildSrc->PathName, "/translate.c\n", NULL);
			build_create_file (VarBuildSrc->PathName, "translate", ".c", GstrTranslate->str);
			GstrTranslate = NULL;
		}
	}
}
// CREATION FICHIER src/main.c
// 
void build_create_SRC_C_file (VAR_BUILD_SRC *VarBuildSrc)
{
	GString	*Gstr = NULL;
	GString	*GstrGlade = NULL;

	Gstr = g_string_new (NULL);
	
	if (TRUE == VarBuild.Bool_lib_gtk) {

		g_string_append_printf (Gstr, "/*\n");
		g_string_append_printf (Gstr, " *  src/main.c\n");
		g_string_append_printf (Gstr, " *  with GTK+-2.0\n");
		g_string_append_printf (Gstr, " */\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "#ifdef HAVE_CONFIG_H\n");
		g_string_append_printf (Gstr, "	#include \"../config.h\"\n");
		g_string_append_printf (Gstr, "#endif\n");
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_po) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
			g_string_append_printf (Gstr, "	#include <libintl.h>\n");
			g_string_append_printf (Gstr, "	#define _(String) gettext (String)\n");
			g_string_append_printf (Gstr, "#endif\n");
		}
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "#include <gtk/gtk.h>\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "#include <stdio.h>\n");
		g_string_append_printf (Gstr, "#include <stdlib.h>\n");
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_lib_statique) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_STATIC_LIBRARY\n");
			g_string_append_printf (Gstr, "	#include \"../%s/lib.h\"\n", VarBuild.NameLib);
			g_string_append_printf (Gstr, "#endif\n");
			g_string_append_printf (Gstr, "\n");
		}
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "GtkBuilder	*GtkBuilderProjet = NULL;\n");
		g_string_append_printf (Gstr, "GtkWidget	*WindMain = NULL;\n");
		g_string_append_printf (Gstr, "\n");
		
		if (TRUE == VarBuild.Bool_import_glade_file) {
		
			GstrGlade = Proto_get_buf_main ();
			g_string_append_printf (Gstr, "%s", GstrGlade->str);
			GstrGlade = NULL;
		}
		else {
			g_string_append_printf (Gstr, "gboolean on_window_main_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)\n");
			g_string_append_printf (Gstr, "{\n");
			if (TRUE == VarBuild.Bool_dossier_po) {
				g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
				g_string_append_printf (Gstr, "	g_print(_(\"End of Program from delete_event\\n\"));\n");
				g_string_append_printf (Gstr, "#endif\n");
			}
			g_string_append_printf (Gstr, "	gtk_main_quit ();\n");
			g_string_append_printf (Gstr, "	return FALSE;\n");
			g_string_append_printf (Gstr, "}\n");
			g_string_append_printf (Gstr, "gboolean on_window_main_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)\n");
			g_string_append_printf (Gstr, "{\n");
			if (TRUE == VarBuild.Bool_dossier_po) {
				g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
				g_string_append_printf (Gstr, "	g_print(_(\"End of Program from destroy_event\\n\"));\n");
				g_string_append_printf (Gstr, "#endif\n");
			}
			g_string_append_printf (Gstr, "	gtk_main_quit ();\n");
			g_string_append_printf (Gstr, "	return FALSE;\n");
			g_string_append_printf (Gstr, "}\n");
			g_string_append_printf (Gstr, "void on_button_quit_clicked (GtkButton *button, gpointer user_data)\n");
			g_string_append_printf (Gstr, "{\n");
			if (TRUE == VarBuild.Bool_dossier_po) {
				g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
				g_string_append_printf (Gstr, "	g_print(_(\"End of Program from button\\n\\n\"));\n");
				g_string_append_printf (Gstr, "#endif\n");
			}
			g_string_append_printf (Gstr, "	gtk_main_quit ();\n");
			g_string_append_printf (Gstr, "}\n");
		}
		
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "/* gint main (gint argc, gchar *argv[], gchar **envp)\n");
		g_string_append_printf (Gstr, " *\n");
		g_string_append_printf (Gstr, " * argc  designe le nombre d'arguments transmis au moment du lancement de l'executable\n");
		g_string_append_printf (Gstr, " * argv  designe le vecteur contenant les differents arguments\n");
		g_string_append_printf (Gstr, " * envp  designe le vecteur contenant les informations sur l'environnement\n");
		g_string_append_printf (Gstr, " */\n");
		g_string_append_printf (Gstr, "gint main (gint argc, gchar *argv[], gchar **envp)\n");
		g_string_append_printf (Gstr, "{\n");
		g_string_append_printf (Gstr, "	GError	*p_err = NULL;\n");
		g_string_append_printf (Gstr, "	gchar	*CurrentDir = g_get_current_dir ();\n");
		g_string_append_printf (Gstr, "	gchar	*PathCurrentDir = NULL;\n");
		g_string_append_printf (Gstr, "	gchar	*PathShareProjet = NULL;\n");
		g_string_append_printf (Gstr, "	gchar	*Path = NULL;\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	g_print (\"%%s :: %%s (line = %%d)\\n\", __FILE__, __FUNCTION__, __LINE__);\n");
		g_string_append_printf (Gstr, "	g_print (\"\\n\");\n");
		g_string_append_printf (Gstr, "	g_print (\"***************************\\n\");\n");
		g_string_append_printf (Gstr, "	g_print (\"%%s %%s\\n\", PACKAGE, VERSION);\n");
		g_string_append_printf (Gstr, "	g_print (\"***************************\\n\");\n");
		g_string_append_printf (Gstr, "	g_print (\"%%s %%s compiled at %%s %%s\\n\", PACKAGE, VERSION, __DATE__, __TIME__);\n");
		g_string_append_printf (Gstr, "	g_print (\"Compiler: gcc %%s\\n\", __VERSION__);\n");
		g_string_append_printf (Gstr, "	g_print (\"***************************\\n\");\n");
		g_string_append_printf (Gstr, "	g_print (\"Gtk version  = %%d.%%d.%%d\\n\", GTK_MAJOR_VERSION, GTK_MICRO_VERSION, GTK_MINOR_VERSION);\n");
		g_string_append_printf (Gstr, "	g_print (\"Glib version = %%d.%%d.%%d\\n\", GLIB_MAJOR_VERSION, GLIB_MICRO_VERSION, GLIB_MINOR_VERSION);\n");
		g_string_append_printf (Gstr, "	g_print (\"***************************\\n\\n\");\n");
		g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
		g_string_append_printf (Gstr, "	g_print (\"ENABLE_NLS OK\\n\");\n");
		g_string_append_printf (Gstr, "	g_print (\"\\tGETTEXT_PACKAGE    = %%s\\n\", GETTEXT_PACKAGE);\n");
		g_string_append_printf (Gstr, "	g_print (\"\\tLOCALE_DIR         = %%s\\n\", LOCALE_DIR);\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	setlocale (LC_ALL, \"\");\n");
		g_string_append_printf (Gstr, "	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);\n");
		g_string_append_printf (Gstr, "	bind_textdomain_codeset (GETTEXT_PACKAGE, \"UTF-8\");\n");
		g_string_append_printf (Gstr, "	textdomain (GETTEXT_PACKAGE);\n");
		g_string_append_printf (Gstr, "#else\n");
		g_string_append_printf (Gstr, "	// g_error (\"ENABLE_NLS IS NOT OK\\n\\n\");\n");
		g_string_append_printf (Gstr, "#endif\n");
		g_string_append_printf (Gstr, "	gtk_set_locale ();\n");
		g_string_append_printf (Gstr, "	gtk_init (&argc, &argv);\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	if (NULL == (GtkBuilderProjet = gtk_builder_new ())) {\n");
		g_string_append_printf (Gstr, "		return (EXIT_FAILURE);\n");
		g_string_append_printf (Gstr, "	}\n");
		g_string_append_printf (Gstr, "	gtk_builder_set_translation_domain (GtkBuilderProjet, NULL);\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	PathCurrentDir = g_strdup_printf (\"%%s/glade/%%s\", CurrentDir, PACKAGE_NAME_GLADE);\n");
		g_string_append_printf (Gstr, "	g_free (CurrentDir);\n");
		g_string_append_printf (Gstr, "	CurrentDir = NULL;\n");
		g_string_append_printf (Gstr, "	PathShareProjet = g_strdup_printf (\"%%s/%%s/glade/%%s\", DATA_DIR, PACKAGE_NAME, PACKAGE_NAME_GLADE);\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	if (FALSE == g_file_test (PathCurrentDir, G_FILE_TEST_EXISTS) && FALSE == g_file_test (PathShareProjet, G_FILE_TEST_EXISTS)) {\n");
		g_string_append_printf (Gstr, "		g_print(\"!---------------------------------------------------------------------------------------------------\\n\");\n");
		g_string_append_printf (Gstr, "		g_print(\"! LE FICHIER XML: [ %%s ] N'EXISTE PAS !!!\\n\", PathCurrentDir);\n");
		g_string_append_printf (Gstr, "		g_print(\"!---------------------------------------------------------------------------------------------------\\n\");\n");
		g_string_append_printf (Gstr, "		g_print(\"! LE FICHIER XML: [ %%s ] N'EXISTE PAS !!!\\n\", PathShareProjet);\n");
		g_string_append_printf (Gstr, "		g_print(\"!---------------------------------------------------------------------------------------------------\\n\");\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		g_free (PathCurrentDir);\n");
		g_string_append_printf (Gstr, "		PathCurrentDir = NULL;\n");
		g_string_append_printf (Gstr, "		g_free (PathShareProjet);\n");
		g_string_append_printf (Gstr, "		PathShareProjet = NULL;\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		return (EXIT_FAILURE);\n");
		g_string_append_printf (Gstr, "	}\n");
		g_string_append_printf (Gstr, "	else if (TRUE == g_file_test (PathCurrentDir, G_FILE_TEST_EXISTS)) {\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		Path = PathCurrentDir;\n");
		g_string_append_printf (Gstr, "	}\n");
		g_string_append_printf (Gstr, "	else if (TRUE == g_file_test (PathShareProjet, G_FILE_TEST_EXISTS)) {\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		Path = PathShareProjet;\n");
		g_string_append_printf (Gstr, "	}\n");
		g_string_append_printf (Gstr, "	g_print(\"\\nPATHNAME FILE %%s: %%s\\n\\n\", PACKAGE_NAME_GLADE, Path);\n");
		g_string_append_printf (Gstr, "	if (gtk_builder_add_from_file (GtkBuilderProjet, Path, &p_err) <= 0) {\n");
		g_string_append_printf (Gstr, "		g_warning (\"Couldn't load builder file: %%s\", p_err->message);\n");
		g_string_append_printf (Gstr, "		g_error_free (p_err);\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		g_free (PathCurrentDir);\n");
		g_string_append_printf (Gstr, "		PathCurrentDir = NULL;\n");
		g_string_append_printf (Gstr, "		g_free (PathShareProjet);\n");
		g_string_append_printf (Gstr, "		PathShareProjet = NULL;\n");
		g_string_append_printf (Gstr, "		Path = NULL;\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "		return (EXIT_FAILURE);\n");
		g_string_append_printf (Gstr, "	}\n");
		g_string_append_printf (Gstr, "	g_free (PathCurrentDir);\n");
		g_string_append_printf (Gstr, "	PathCurrentDir = NULL;\n");
		g_string_append_printf (Gstr, "	g_free (PathShareProjet);\n");
		g_string_append_printf (Gstr, "	PathShareProjet = NULL;\n");
		g_string_append_printf (Gstr, "	Path = NULL;\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	gtk_builder_set_translation_domain  (GtkBuilderProjet, gtk_set_locale ());\n");
		g_string_append_printf (Gstr, "	gtk_builder_connect_signals (GtkBuilderProjet, NULL);\n");
		
		if (TRUE == VarBuild.Bool_import_glade_file) {
			g_string_append_printf (Gstr, "	WindMain = GTK_WIDGET (gtk_builder_get_object (GtkBuilderProjet, \"%s\"));\n", Proto_get_id ());
		}
		else {
			g_string_append_printf (Gstr, "	WindMain = GTK_WIDGET (gtk_builder_get_object (GtkBuilderProjet, \"window_main\"));\n");
		}
		
		g_string_append_printf (Gstr, "	gtk_widget_show (WindMain);\n");
		if (TRUE == VarBuild.Bool_dossier_po) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
			g_string_append_printf (Gstr, "\tprintf (_(\"Chaine a traduire\\n\"));\n");
			g_string_append_printf (Gstr, "#endif\n");
		}
		g_string_append_printf (Gstr, "	gtk_main ();\n");
		g_string_append_printf (Gstr, "\n");
		g_string_append_printf (Gstr, "	exit(EXIT_SUCCESS);\n");
		g_string_append_printf (Gstr, "}\n");
		g_string_append_printf (Gstr, "\n");

		build_create_file (VarBuildSrc->PathName, "main", ".c", Gstr->str);
	
	}
	else {
	
		g_string_append_printf (Gstr, "/*\n");
		g_string_append_printf (Gstr, "* file      : %s/main.c\n", VarBuildSrc->Name);
		g_string_append_printf (Gstr, "* project   : %s\n", VarBuild.NameProject);
		g_string_append_printf (Gstr, "* without   : Gtk-2\n");
		g_string_append_printf (Gstr, " */\n");
		g_string_append_printf (Gstr, "\n");
		
		g_string_append_printf (Gstr, "#ifdef HAVE_CONFIG_H\n");
		g_string_append_printf (Gstr, "	#include \"../config.h\"\n");
		g_string_append_printf (Gstr, "#endif\n");
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_po) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
			g_string_append_printf (Gstr, "	#include <libintl.h>\n");
			g_string_append_printf (Gstr, "	#define _(String) gettext (String)\n");
			g_string_append_printf (Gstr, "#endif\n");
			g_string_append_printf (Gstr, "\n");
		}
		g_string_append_printf (Gstr, "#include <stdio.h>\n");
		g_string_append_printf (Gstr, "#include <stdlib.h>\n");
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_lib_statique) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_STATIC_LIBRARY\n");
			g_string_append_printf (Gstr, "	#include \"../%s/lib.h\"\n", VarBuild.NameLib);
			g_string_append_printf (Gstr, "#endif\n");
			g_string_append_printf (Gstr, "\n");
		}
		g_string_append_printf (Gstr, "/* int main (int argc, char *argv[], char **envp)\n");
		g_string_append_printf (Gstr, " *\n");
		g_string_append_printf (Gstr, " * argc  designe le nombre d'arguments transmis au moment du lancement de l'executable\n");
		g_string_append_printf (Gstr, " * argv  designe le vecteur contenant les differents arguments\n");
		g_string_append_printf (Gstr, " * envp  designe le vecteur contenant les informations sur l'environnement\n");
		g_string_append_printf (Gstr, " */\n");
		g_string_append_printf (Gstr, "int main (int argc, char *argv[], char **envp)\n");
		g_string_append_printf (Gstr, "{\n");
		g_string_append_printf (Gstr, "\tprintf (\"%%s :: %%s (line = %%d)\\n\", __FILE__, __FUNCTION__, __LINE__);");
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_po) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_NLS\n");
			g_string_append_printf (Gstr, "\tprintf (_(\"Chaine a traduire depuis %s/\\n\"));\n", VarBuildSrc->Name);
			g_string_append_printf (Gstr, "#endif\n");
		}
		g_string_append_printf (Gstr, "\n");
		if (TRUE == VarBuild.Bool_dossier_lib_statique) {
			g_string_append_printf (Gstr, "#ifdef ENABLE_STATIC_LIBRARY\n");
			g_string_append_printf (Gstr, "\tlibH_print_mess (\"Appel fonction librairie statique depuis %s = ok\\n\");\n", VarBuildSrc->Name);
			g_string_append_printf (Gstr, "#endif\n");
		}
		g_string_append_printf (Gstr, "\texit(EXIT_SUCCESS);\n");
		g_string_append_printf (Gstr, "}\n");
		g_string_append_printf (Gstr, "\n");
				
		build_create_file (VarBuildSrc->PathName, "main", ".c", Gstr->str);
	}
	
	g_string_free (Gstr, TRUE);
}
// CREATION FICHIER man/NOM_PROJET.1
// 
void build_create_MAN_file (void)
{
	GString	*Gstr = NULL;
	gchar	*NameManZip = NULL;
	gchar	*LineCommand = NULL;
 	gint	 RetSystem;
      
	// CONSTRUCTION DU FICHIER TEXTE MANPAGE
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, ".\\\" %s_man\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, ".\\\" Copyright (C) 2010\n");
	g_string_append_printf (Gstr, ".\\\"\n");
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, ".TH %s 1 \"Sat, 01 May 2010\" %s \"Page d'aide de: %s\"\n", VarBuild.NameProject, VarBuild.VersionProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, ".SH NOM\n");
	g_string_append_printf (Gstr, "\\fB%s\\fP \\- GTK+ Implementation of the GNU.\n", VarBuild.NameProject);
	
	// ENREGISTREMENT DU FICHIER TEXTE MANPAGE
	NameManZip = g_strdup_printf ("%s.1", VarBuild.NameProject);
	build_create_file (VarBuild.PathNameMan, NameManZip, "", Gstr->str);
	g_string_free (Gstr, TRUE);
	Gstr = NULL;
	g_free (NameManZip);
	NameManZip = NULL;
	
	// CONSTRUCTION DU FICHIER ZIP MANPAGE
	NameManZip = g_strdup_printf ("%s/%s.1", VarBuild.PathNameMan, VarBuild.NameProject);
	LineCommand = g_strdup_printf ("gzip --best %s", NameManZip);
	RetSystem = system (LineCommand);
	g_free (LineCommand);
	LineCommand = NULL;
	g_free (NameManZip);
	NameManZip = NULL;
	
	// FICHIER D AIDE
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, "\n");
	g_string_append_printf (Gstr, "gzip --best ./%s.1\n\n", VarBuild.NameProject);
	build_create_file (VarBuild.PathNameMan, "compilate_manpage", ".txt", Gstr->str);
	g_string_free (Gstr, TRUE);
	Gstr = NULL;
}
// CREATION FICHIER man/NOM_PROJET.1.gz
// 
void build_create_MENU_file (void)
{
	GString	*Gstr = NULL;

	// CREATION FICHIER menu/NOM_PROJET.desktop
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, "[Desktop Entry]\n");
	g_string_append_printf (Gstr, "Version=%s\n", VarBuild.VersionProject);
	g_string_append_printf (Gstr, "Name=%s\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "Name[fr]=%s\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "GenericName[fr]=%s\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "Comment=%s. My new application with basebuild.\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "Icon=test%s.png\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "Type=Application\n");
	g_string_append_printf (Gstr, "Exec=%s\n", VarBuild.NameProject);
	g_string_append_printf (Gstr, "Terminal=false\n");
	g_string_append_printf (Gstr, "StartupNotify=true\n");
	g_string_append_printf (Gstr, "%s", Callbacks_get_entry_categories_item ());
	//	
	//	http://standards.freedesktop.org/menu-spec/1.0/apa.html
	//	AudioVideo
	//	Audio
	//	Video
	//	Development
	//	Education
	//	Game
	//	Graphics
	//	Network
	//	Office
	//	Settings
	//	System
	//	Utility
	
	build_create_file (VarBuild.PathNameMenu, VarBuild.NameProject, ".desktop", Gstr->str);
	g_string_free (Gstr, TRUE);
}
// CREATION FICHIERS glade/NOM_PROJET.glade
// 
void build_create_GLADE_file (void)
{
	gchar *BufGlade = 
		"<?xml version=\"1.0\"?>\n"
		"<interface>\n"
		"  <requires lib=\"gtk+\" version=\"2.16\"/>\n"
		"  <!-- interface-naming-policy project-wide -->\n"
		"  <object class=\"GtkWindow\" id=\"window_main\">\n"
		"    <property name=\"width_request\">150</property>\n"
		"    <property name=\"height_request\">100</property>\n"
		"    <property name=\"window_position\">center</property>\n"
		"    <signal name=\"destroy_event\" handler=\"on_window_main_destroy_event\"/>\n"
		"    <signal name=\"delete_event\" handler=\"on_window_main_delete_event\"/>\n"
		"    <child>\n"
		"      <object class=\"GtkVBox\" id=\"vbox1\">\n"
		"        <property name=\"visible\">True</property>\n"
		"        <property name=\"orientation\">vertical</property>\n"
		"        <child>\n"
		"          <object class=\"GtkLabel\" id=\"label1\">\n"
		"            <property name=\"visible\">True</property>\n"
		"            <property name=\"label\" translatable=\"yes\">&lt;b&gt;BASEBUILD&lt;/b&gt;</property>\n"
		"            <property name=\"use_markup\">True</property>\n"
		"          </object>\n"
		"          <packing>\n"
		"            <property name=\"position\">0</property>\n"
		"          </packing>\n"
		"        </child>\n"
		"        <child>\n"
		"          <object class=\"GtkButton\" id=\"button_quit\">\n"
		"            <property name=\"label\" translatable=\"yes\">quit</property>\n"
		"            <property name=\"visible\">True</property>\n"
		"            <property name=\"can_focus\">True</property>\n"
		"            <property name=\"receives_default\">True</property>\n"
		"            <signal name=\"clicked\" handler=\"on_button_quit_clicked\"/>\n"
		"          </object>\n"
		"          <packing>\n"
		"            <property name=\"expand\">False</property>\n"
		"            <property name=\"position\">1</property>\n"
		"          </packing>\n"
		"        </child>\n"
		"      </object>\n"
		"    </child>\n"
		"  </object>\n"
		"</interface>\n"
		"\n";

	if (TRUE == Proto_init_is_ok ()) {
		gchar	*Src = Callbacks_get_pathnamefile_glade ();
		gchar	*Dest = g_strdup_printf ("%s/%s.glade", VarBuild.PathNameGlade, VarBuild.NameProject);
		
		Proto_copy_glade_file (Src, Dest);
		
		g_free (Src);
		Src = NULL;
		g_free (Dest);
		Dest = NULL;
	}
	else {
		build_create_file (VarBuild.PathNameGlade, VarBuild.NameProject, ".glade", BufGlade);
	}
	
}
// CREATION FICHIER po/fr.po
// 
void build_create_PO_file (gchar *PathNamaneFile_PO)
{
	GString	*Gstr = NULL;
	FILE	*fp = NULL;
	gchar	*PathName = NULL;
	gchar	*NamePo[] = {"en.po", "fr.po", "de.po", "it.po" };
	gchar	*Language[] = {"English", "French", "German", "Italy" };
	gint	Cpt;
	
	for (Cpt = 0; Cpt < 4; Cpt ++) {
		Gstr = g_string_new (NULL);
		g_string_append_printf (Gstr, "# SOME DESCRIPTIVE TITLE.\n");
		g_string_append_printf (Gstr, "# Copyright (C) YEAR Free Software Foundation, Inc.\n");
		g_string_append_printf (Gstr, "# This file is distributed under the same license as the PACKAGE package.\n");
		g_string_append_printf (Gstr, "# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n");
		g_string_append_printf (Gstr, "#\n");
		// g_string_append_printf (Gstr, "#, fuzzy\n");
		g_string_append_printf (Gstr, "msgid \"\"\n");
		g_string_append_printf (Gstr, "msgstr \"\"\n");
		g_string_append_printf (Gstr, "\"Project-Id-Version: %s %s\\n\"\n", VarBuild.NameProject, VarBuild.VersionProject);
		g_string_append_printf (Gstr, "\"Report-Msgid-Bugs-To: Emailr@Adress.Ext\\n\"\n");
		g_string_append_printf (Gstr, "\"POT-Creation-Date: 2010-05-06 16:59+0200\\n\"\n");
		g_string_append_printf (Gstr, "\"PO-Revision-Date: 2010-05-06 16:59+0200\\n\"\n");
		g_string_append_printf (Gstr, "\"Last-Translator: MyName <Emailr@Adress.Ext>\\n\"\n");
		g_string_append_printf (Gstr, "\"Language-Team: %s\\n\"\n", Language [ Cpt ]);
		g_string_append_printf (Gstr, "\"MIME-Version: 1.0\\n\"\n");
		g_string_append_printf (Gstr, "\"Content-Type: text/plain; charset=UTF-8\\n\"\n");
		g_string_append_printf (Gstr, "\"Content-Transfer-Encoding: 8bit\\n\"\n");
		
		PathName = g_strdup_printf ("%s/%s", PathNamaneFile_PO, NamePo [ Cpt ]);
		if (NULL != (fp = fopen (PathName, "w"))) {
			g_fprintf (fp, "%s", Gstr->str);
			fclose (fp);
		}
		g_free (PathName);
		PathName = NULL;
		
		g_string_free (Gstr, TRUE);
	}
}
// COPIE FICHIERS /usr/share/pixmaps/$VarBuild.NameProject/testbasebuild.png
// 
gboolean build_create_PIXMAPS_file (void)
{
	gchar	*CurrentDir = g_get_current_dir ();
	gchar	*PathCurrentDir = NULL;
	gchar	*LineCommand = NULL;
	gint	 RetSystem;
	
	// TEST SOURCE FILE
	PathCurrentDir = g_strdup_printf ("%s/pixmaps/testbasebuild.png", CurrentDir);
	if (FALSE == libutils_test_file_exist (PathCurrentDir)) {
		g_free (PathCurrentDir);
		PathCurrentDir = NULL;
		PathCurrentDir = g_strdup_printf ("%s/pixmaps/testbasebuild.png", DATA_DIR);
		if (FALSE == libutils_test_file_exist (PathCurrentDir)) {
			g_free (PathCurrentDir);
			PathCurrentDir = NULL;
			return (FALSE);
		}
	}
	
	// SOURCE IN: PathCurrentDir
	// DEST IN:   VarBuild.PathNamePixmaps
	LineCommand = g_strdup_printf ("cp %s %s/test%s.png", PathCurrentDir, VarBuild.PathNamePixmaps, VarBuild.NameProject);
	RetSystem = system (LineCommand);
	g_free (LineCommand);
	LineCommand = NULL;
	g_free (CurrentDir);
	CurrentDir = NULL;
	
	return (TRUE);
}
// CREATION FICHIER mime/$VarBuild.NameProject
// 
void build_create_MIME_file (void)
{
	GString	*Gstr = NULL;
	
	Gstr = g_string_new (NULL);
	g_string_append_printf (Gstr, "application/ogg; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/basic; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/mpeg; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/x-aiff; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/x-mp3; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/x-wav; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "application/x-flac; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	g_string_append_printf (Gstr, "audio/x-flac; %s '%%%%s'; edit=%s '%%%%s'; test=test \"$DISPLAY\" != \"\"; priority=2\n", VarBuild.NameProject, VarBuild.NameProject);
	build_create_file (VarBuild.PathNameMime, VarBuild.NameProject, "", Gstr->str);
	g_string_free (Gstr, TRUE);
}
// 
// 
void build_create_GETTEXTIZE_file (void)
{
	GString	*Gstr = NULL;
	gint	 RetSystem;
	
	Gstr = g_string_new (NULL);
	
	g_string_append_printf (Gstr, "#!/bin/sh\n");
	g_string_append_printf (Gstr, "PROGRAM=gettextize\n");
	g_string_append_printf (Gstr, "sed 's:read .*< /dev/tty::' `which $PROGRAM` > /tmp/my-gettextize.sh\n");
	g_string_append_printf (Gstr, "chmod +x /tmp/my-gettextize.sh\n");
	g_string_append_printf (Gstr, "exit 0\n");
	
	build_create_file ("/tmp", "set-gettextize.sh", "", Gstr->str);
	RetSystem = system ("chmod +x /tmp/set-gettextize.sh");
	RetSystem = system ("/tmp/set-gettextize.sh");
}
// GENERATION DU PROJET DEPUIS UN THREAD
// 


extern VAR_PROTO VarProto;

static void build_pthread_clicked (void *arg)
{
	gchar		*LineCommand = NULL;
	gchar		*TmpActuel = g_get_current_dir ();
	FILE		*fp = NULL;
	gchar		*PathNamePotfile = NULL;
	gint		RetSystem = -2;
	GList		*List = NULL;
	VAR_BUILD_SRC	*VarBuildSrc = NULL;
	
	// PRINT_FUNC_LF();
	
	VarBuild.bool_end_thread = FALSE;

	// CE BUFFER EST UTILE POUR L'AFFICHAGE DE LA PARTIE translate DANS LE TREVIEW
	if (NULL != Proto_get_buf_translate ()) {
		g_string_free (VarProto.GstrTranslate, TRUE);
		VarProto.GstrTranslate = NULL;
	}
	build_maj_options ();
	if (NULL != Proto_get_buf_translate ()) {
		wiew_set_text_translate (VarProto.GstrTranslate->str);
	}
	
	view_set_text_va ("\nCREATION DU PROJET  [ ", VarBuild.NameProject, "-", VarBuild.VersionProject, " ]\n\n", NULL);
	
	// SUPPRESSION DU DOSSIER SI IL EXISTE
	if (g_file_test (VarBuild.NewPathName, G_FILE_TEST_IS_DIR)) {
		if (NULL != (LineCommand = g_strdup_printf ("rm -rf %s", VarBuild.NewPathName))) {
			view_set_text_va ("SUPPRESSION DE L ANCIEN DOSSIER:\n\t", VarBuild.NewPathName, "\n", NULL);
			RetSystem = system (LineCommand);
			g_free (LineCommand);
			LineCommand = NULL;
		}
	}
	
	// CREATION DOSSIER DU PROJET
	view_set_text_va ("CREATION DOSSIER DU PROJET:\n\t", VarBuild.NewPathName, "\n", NULL);
	g_mkdir_with_parents (VarBuild.NewPathName, 0700);
	
	// CREATION FICHIER Makefile.am
	BuildBuf_create_makefile_am ();
	view_set_text_va ("\t", VarBuild.NewPathName, "/Makefile.am\n", NULL);
	build_create_file (VarBuild.NewPathName, "Makefile", ".am", VarBuild.GstrMakefileAm->str);

	// LES DOSSIERS LIBRAIRIE STATIQUE
	if (TRUE == VarBuild.Bool_dossier_lib_statique) {
		view_set_text_va ("CREATION DOSSIER LIBRAIRIE STATIQUE:\n\t", VarBuild.PathNameLib, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNameLib, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_lib_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNameLib, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNameLib, "Makefile", ".am", VarBuild.GstrLibMakefileAm->str);
		// CREATION FICHIER lib/lib.c lib/lib.h
		view_set_text_va ("\t", VarBuild.PathNameLib, "/lib.c\n", NULL);
		view_set_text_va ("\t", VarBuild.PathNameLib, "/lib.h\n", NULL);
		build_create_LIB_C_file (VarBuild.PathNameLib);
	}

	// 
	BuildBuf_create_src_all_makefile_am ();
	// CREATE DOSSIER SRC
	List = g_list_first (VarBuild.ListPathSrc);
	while (List) {
		if (NULL != (VarBuildSrc = (VAR_BUILD_SRC *)List->data)) {
			// CREATE DOSSIER SRC	
			view_set_text_va ("CREATION DOSSIER SOURCE:\n\t", VarBuildSrc->PathName, "\n", NULL);
			g_mkdir_with_parents (VarBuildSrc->PathName, 0700);
			// CREATION FICHIER Makefile.am
			view_set_text_va ("\t", VarBuildSrc->PathName, "/Makefile.am\n", NULL);
			build_create_file (VarBuildSrc->PathName, "Makefile", ".am", VarBuildSrc->GstrSrcMakefileAm->str);
			// CREATION FICHIER src.../main.c
			view_set_text_va ("\t", VarBuildSrc->PathName, "/main.c\n", NULL);
			build_create_SRC_C_file (VarBuildSrc);
			build_create_SRC_TRANSLATE_C_file (VarBuildSrc);
		}
		List = g_list_next (List);
	}
	if (TRUE == VarBuild.Bool_dossier_po) {
		view_set_text_va ("CREATION DOSSIER INTERNATIONALISATION:\n\t", VarBuild.PathNamePo, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNamePo, 0700);
	}
	if (TRUE == VarBuild.Bool_dossier_manpage) {
		view_set_text_va ("CREATION DOSSIER MANPAGE:\n\t", VarBuild.PathNameMan, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNameMan, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_man_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNameMan, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNameMan, "Makefile", ".am", VarBuild.GstrManMakefileAm->str);
		// CREATION FICHIER man.1.gz
		view_set_text_va ("\t", VarBuild.PathNameMan, "/", VarBuild.NameProject, ".1.gz\n", NULL);
		build_create_MAN_file ();
	}
	if (TRUE == VarBuild.Bool_dossier_pixmaps) {
		view_set_text_va ("CREATION DOSSIER PIXMAPS:\n\t", VarBuild.PathNamePixmaps, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNamePixmaps, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_pixmaps_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNamePixmaps, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNamePixmaps, "Makefile", ".am", VarBuild.GstrPixmapsMakefileAm->str);
		// CREATION FICHIER pixmaps/testbasebuild.png
		view_set_text_va ("\t", VarBuild.PathNamePixmaps, "/test", VarBuild.NameProject, ".png\n", NULL);
		build_create_PIXMAPS_file ();
	}
	if (TRUE == VarBuild.Bool_dossier_menu) {
		view_set_text_va ("CREATION DOSSIER MENU:\n\t", VarBuild.PathNameMenu, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNameMenu, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_menu_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNameMenu, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNameMenu, "Makefile", ".am", VarBuild.GstrMenuMakefileAm->str);
		// CREATION FICHIER menu.menu
		view_set_text_va ("\t", VarBuild.PathNameMenu, "/", VarBuild.NameProject, ".desktop\n", NULL);
		build_create_MENU_file ();
	}
	if (TRUE == VarBuild.Bool_dossier_mime) {
		view_set_text_va ("CREATION DOSSIER MIME:\n\t", VarBuild.PathNameMime, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNameMime, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_mime_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNameMime, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNameMime, "Makefile", ".am", VarBuild.GstrMimeMakefileAm->str);
		// CREATION FICHIER mime.mime
		view_set_text_va ("\t", VarBuild.PathNameMime, "/", VarBuild.NameProject, "\n", NULL);
		build_create_MIME_file ();
	}
	if (TRUE == VarBuild.Bool_dossier_glade) {
		view_set_text_va ("CREATION DOSSIER GLADE:\n\t", VarBuild.PathNameGlade, "\n", NULL);
		g_mkdir_with_parents (VarBuild.PathNameGlade, 0700);
		// CREATION FICHIER Makefile.am
		BuildBuf_create_glade_makefile_am ();
		view_set_text_va ("\t", VarBuild.PathNameGlade, "/Makefile.am\n", NULL);
		build_create_file (VarBuild.PathNameGlade, "Makefile", ".am", VarBuild.GstrGladeMakefileAm->str);
		// CREATION FICHIER glade.glade
		view_set_text_va ("\t", VarBuild.PathNameGlade, "/", VarBuild.NameProject, ".glade\n", NULL);
		build_create_GLADE_file ();
	}
	
	// SE PLACER DANS LE DOSSIER VarBuild.NewPathName/
	g_chdir (VarBuild.NewPathName);
		
	// CREATION FICHIER configure.in
	BuildBuf_create_configure_in ();
	view_set_text_va ("CREATION FICHIER:\n\t", VarBuild.NewPathName, "/configure.in\n", NULL);
	build_create_file (VarBuild.NewPathName, "configure", ".in", VarBuild.GstrConfigureIn->str);
	
	if (TRUE == VarBuild.Bool_dossier_po) {
				
		// POUR OUTREPASSER GETTEXTIZE
		// http://culot.org/public/Docs/autotools.html
		view_set_text_va ("OUTREPASSER\n\tgettextize AVEC /tmp/my-gettextize.sh\n", NULL);
		build_create_GETTEXTIZE_file ();
		
		// gettextize copie tous les fichiers standards Gettext dans le répertoire principal d'un package, pour commencer sa traduction.
		// Prise en compte de l'internationalisation qui modifie 'configure.in' et 'Makefile.am'
		// 	$ gettextize -c
		// Le fichier ./ABOUT-NLS sera cree
		view_set_text_va ("EXEC\n\t/tmp/my-gettextize.sh -c\n", NULL);
		if (NULL != (LineCommand = g_strdup ("/tmp/my-gettextize.sh -c"))) {
			RetSystem = system (LineCommand);
			g_free (LineCommand);
			LineCommand = NULL;
		}
		
		view_set_text_va ("SUPPRESSION DE:\n\t/tmp/set-gettextize.sh\n\t/tmp/my-gettextize.sh\n", NULL);
		g_unlink ("/tmp/set-gettextize.sh");
		g_unlink ("/tmp/my-gettextize.sh");
		
		// RENOMMER 'Makevars.template' vers 'po/Makevars'
		{
		gchar	*OldFile = g_strdup_printf ("%s/Makevars.template", VarBuild.PathNamePo);
		gchar	*NewFile = g_strdup_printf ("%s/Makevars", VarBuild.PathNamePo);
		view_set_text_va ("RENOMMER:\n\t", OldFile, "\n\t", NewFile, "\n", NULL);
		g_rename(OldFile, NewFile);
		g_free (OldFile);
		OldFile = NULL;
		g_free (NewFile);
		NewFile = NULL;
		}
		
		// ADAPTATION DE 'po/POTFILES.in'
		{
		GString	*GstrPot = NULL;
		GList	*List = NULL;
		
		PathNamePotfile = g_strdup_printf ("%s/POTFILES.in", VarBuild.PathNamePo);
		if (NULL != (fp = fopen (PathNamePotfile,"a"))) {
			view_set_text_va ("AJOUTER A:\n\t", PathNamePotfile, "\n", NULL);
			GstrPot = g_string_new (NULL);
			List = g_list_first (VarBuild.ListPathSrc);
			while (List) {
				if (NULL != (VarBuildSrc = (VAR_BUILD_SRC *)List->data)) {
					g_string_append_printf (GstrPot, "%s/main.c\n", VarBuildSrc->Name);
					view_set_text_va ("\t", VarBuildSrc->Name, "/main.c\n", NULL);
					if (NULL != Proto_get_buf_translate ()) {
						g_string_append_printf (GstrPot, "%s/translate.c\n", VarBuildSrc->Name);
						view_set_text_va ("\t", VarBuildSrc->Name, "/translate.c\n", NULL);
					}
				}
				List = g_list_next (List);
			}
			g_fprintf (fp, "%s", GstrPot->str);
			fclose (fp);
			
		}
		g_string_free (GstrPot, TRUE);
		}

		// RE CREATION FICHIER configure.in
		build_create_file (VarBuild.NewPathName, "configure", ".in", VarBuild.GstrConfigureIn->str);
		
		// RE CREATION FICHIER Makefile.am
		build_create_file (VarBuild.NewPathName, "Makefile", ".am", VarBuild.GstrMakefileAm->str);
				
		// CREATION
		// 	po/en.po
		//	po/fr.po
		//	po/de.po
		//	po/it.po
		PathNamePotfile = g_strdup_printf ("%s/en.po\n\t%s/fr.po\n\t%s/de.po\n\t%s/it.po\n\t", VarBuild.PathNamePo, VarBuild.PathNamePo, VarBuild.PathNamePo, VarBuild.PathNamePo);
		view_set_text_va ("CREATION FICHIER:\n\t", PathNamePotfile, "\n", NULL);
		g_free (PathNamePotfile);
		PathNamePotfile = NULL;	
			
		build_create_PO_file (VarBuild.PathNamePo);
	}
	
	// CREATION FICHIERS ./NEWS ./README ./AUTHORS ./ChangeLog
	view_set_text_va ("CREATION FICHIERS:\n\t",
					VarBuild.NewPathName, "/NEWS\n\t",
					VarBuild.NewPathName, "/README\n\t",
					VarBuild.NewPathName, "/AUTHORS\n\t",
					VarBuild.NewPathName, "/ChangeLog\n",
					NULL);
	build_create_file (VarBuild.NewPathName, "NEWS", "", NULL);
	build_create_file (VarBuild.NewPathName, "README", "", NULL);
	build_create_file (VarBuild.NewPathName, "AUTHORS", "", NULL);
	build_create_file (VarBuild.NewPathName, "ChangeLog", "", NULL);
		
	// CONSTRUCTION DU PAQUET
	view_set_text_va ("EXEC\n\tautoscan\n", NULL);
	RetSystem = system ("autoscan");	
	view_set_text_va ("\taclocal\n", NULL);
	RetSystem = system ("aclocal");
	view_set_text_va ("\tautoheader\n", NULL);
	RetSystem = system ("autoheader");
	view_set_text_va ("\tautoconf\n", NULL);
	RetSystem = system ("autoconf");
	view_set_text_va ("\tautomake -a -c\n", NULL);
	RetSystem = system ("automake -a -c");
	view_set_text_va ("\tautomake --add-missing\n", NULL);
	RetSystem = system ("automake --add-missing");
	view_set_text_va ("\tautomake\n", NULL);
	RetSystem = system ("automake");
	view_set_text_va ("\t./configure --prefix=/usr\n", NULL);
	RetSystem = system ("./configure --prefix=/usr");
	view_set_text_va ("\tmake\n", NULL);
	RetSystem = system ("make");
	view_set_text_va ("\tmake distcheck\n", NULL);
	RetSystem = system ("make distcheck");

	// SE REPLACER DANS LE DOSSIER D ORIGINE
	g_chdir (TmpActuel);
	g_free (TmpActuel);
	TmpActuel = NULL;
	
	VarBuild.bool_end_thread = TRUE;
	pthread_exit(0);
}
// 
// 
static gint build_timeout (gpointer data)
{
	if (TRUE == VarBuild.bool_end_thread) {
		// ACTIVATION APRES LA GENERATION DU PROJET
		gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("menubar1")), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("frame1")), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("frame2")), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("button_appliquer")), TRUE);
		
		// FREE ALLOCATIONS
		g_print ("\n");
		
		view_set_text_va ("\n********************\n",
				  _("VOTRE PAQUET EST DISPONIBLE DEPUIS LE DOSSIER:"),
				  "\n\t",
				  VarBuild.NewPathName,
				  "\n",
				  NULL);
		if (TRUE == ProtoTranslate_contains_accents ()) {
			view_set_text_va ("\n",
					  "\t",
					  _("*** ERREUR A CORRIGER DANS LE FICHIER DE TRADUCTION:"),
					  "\n\t",
					  _("***   IL CONTIENT DES ACCENTS"),
					  "\n",
					  NULL);
		}
		view_set_text_va ("\n********************\n",
				  NULL);
		
		
		Proto_remove ();
		build_remove ();
		
		gtk_timeout_remove (VarBuild.handler_timeout);
	}
	return (TRUE);
}
// APPEL A THREAD POUR LA GENERATION DU PROJET
// 
void build_clicked (void)
{
	pthread_t	 nmr_tid;
	GString		*Gstr = g_string_new (NULL);
	gboolean	 BoolErr = FALSE;
	
	// PRINT_FUNC_LF();
	
	view_clear_text ();
	
	switch (DefSrc_is_dupply ()) {
	case 0 :
		break;
	case 1 :
		wiew_without_threads ();
		view_set_text_va (_("Mauvaise orthographe !\n\tLe nom n'est pas au standard GNU\n\tVeuillez corriger\n"), NULL);
		gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 1);
		return;
	case 2 :
		wiew_without_threads ();
		view_set_text_va (_("Dupplication de noms de dossiers sources !\n\tLes noms des dossiers sources ne doivent pas etre identiques\n\tVeuillez corriger\n"), NULL);
		gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 1);
		return;
	case 3 :
		wiew_without_threads ();
		view_set_text_va (_("Dupplication de noms de dossiers ET mauvaise orthographe !\n\tVeuillez corriger\n"), NULL);
		gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 1);
		return;
	}
		
	// TEST PRESENCE DES PROGRAMMES EXTERNES
	if (FALSE == libutils_find_file ("gettextize") && TRUE == VarBuild.Bool_dossier_po) {
		g_string_append_printf (Gstr, "\t<b>gettextize</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("autoscan")) {
		g_string_append_printf (Gstr, "\t<b>autoscan</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("aclocal")) {
		g_string_append_printf (Gstr, "\t<b>aclocal</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("autoheader")) {
		g_string_append_printf (Gstr, "\t<b>autoheader</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("autoconf")) {
		g_string_append_printf (Gstr, "\t<b>autoconf</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("automake")) {
		g_string_append_printf (Gstr, "\t<b>automake</b>\n");
		BoolErr = TRUE;
	}
	if (FALSE == libutils_find_file ("make")) {
		g_string_append_printf (Gstr, "\t<b>make</b>\n");
		BoolErr = TRUE;
	}
	if (TRUE == BoolErr) {
		build_remove ();
		wininfo_create (
			_("Applications absentes !"),
			_("   Applications a installer :            "),
			  "\n\n",
			  Gstr->str,
			  "\n",
			  "");
		
		g_string_free (Gstr, TRUE);
		Gstr = NULL;
		return;
	}
	g_string_free (Gstr, TRUE);
	Gstr = NULL;
	
	// DEACTIVATION AVANT LA GENERATION DU PROJET
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("menubar1")), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("frame1")), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("frame2")), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("button_appliquer")), FALSE);
	
 	// THREAD
	VarBuild.bool_end_thread = FALSE;
	pthread_create (&nmr_tid, NULL ,(void *)build_pthread_clicked, (void *)NULL);
	VarBuild.handler_timeout = gtk_timeout_add (300, build_timeout, 0);
}


