 /*
 * file      : main.c
 * project   : BaseBuild
 * with      : Gtk-2
 *
 * copyright : (C) 2010 by Claude Bulin
 *
 * BaseBuild - Creation d'une base de programmation en langage C de type GNU avec les autotools
 * GNU General Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * OLD ADRESS:
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * NEW ADRESS:
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

#include "../libbasebuild/libbasebuild.h"
#include "global.h"
#include "configuser.h"
#include "callbacks.h"
#include "view.h"


GtkBuilder	*GtkBuilderXcfa = NULL;
GtkWidget	*WindMain = NULL;



gint main (gint argc, gchar *argv[], gchar **envp)
{
	GError	*p_err = NULL;
	gchar	*CurrentDir = g_get_current_dir ();
	gchar	*Path = NULL;
	gchar	*PathCurrentDir = NULL;
	gchar	*PathShareXcfa = NULL;
	gchar	*PackageVersion = NULL;
	
	g_print ("\n");
	g_print ("***************************\n");
	g_print ("%s %s\n", PACKAGE, VERSION);
	g_print ("***************************\n");
	g_print ("%s %s compiled at %s %s\n", PACKAGE, VERSION, __DATE__, __TIME__);
	g_print ("Compiler: gcc %s\n", __VERSION__);
	g_print ("***************************\n");
	g_print ("Gtk version  = %d.%d.%d\n", GTK_MAJOR_VERSION, GTK_MICRO_VERSION, GTK_MINOR_VERSION);
	g_print ("Glib version = %d.%d.%d\n", GLIB_MAJOR_VERSION, GLIB_MICRO_VERSION, GLIB_MINOR_VERSION);
	g_print ("***************************\n\n");
	
#ifdef ENABLE_NLS
	g_print ("ENABLE_NLS OK\n");
	g_print ("\tGETTEXT_PACKAGE    = %s\n", GETTEXT_PACKAGE);
	g_print ("\tPACKAGE_LOCALE_DIR = %s\n", PACKAGE_LOCALE_DIR);
	g_print ("\tLOCALEDIR          = %s\n", LOCALEDIR);
	g_print ("\tLOCALE_DIR         = %s\n", LOCALE_DIR);

	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#else
	g_error ("ENABLE_NLS IS NOT OK\n\n");
	exit(EXIT_FAILURE);
#endif
	g_print ("\tDATA_DIR           = %s\n", DATA_DIR);
	g_print ("\tPACKAGE_DATA_DIR   = %s\n", PACKAGE_DATA_DIR);
	g_print ("\n");
	
	config_read();
	
	gtk_set_locale ();
	
	// THREAD INIT
	g_thread_init (NULL);
	if (g_thread_supported()) {
		g_print ("g_thread_supported: YES\n\n");
	} else {
		g_print ("g_thread_supported: NO\n");
		g_print ("Desole les thread ne sont pas supportes pas votre plate-forme.\n"
			 "Il vous faut un systeme linux base sur la libc5 ainsi que GLIB & GTK+\n\n");

		g_print ("Sorry, threads isn't supported on your platform.\n"
			 "If you're on a libc5 based linux system and installed GLIB & GTK+ before you\n"
			 "installed LinuxThreads you need to recompile GLIB & GTK+\n\n");

		g_error ("\n");
	}
	gdk_threads_init ();
	gtk_init (&argc, &argv);
	gdk_threads_enter ();
	
	if (NULL == (GtkBuilderXcfa = gtk_builder_new ())) {
		// THREAD END
		gdk_threads_leave ();
		return (EXIT_FAILURE);
	}
	gtk_builder_set_translation_domain (GtkBuilderXcfa, NULL);
	
	PathCurrentDir = g_strdup_printf ("%s/glade/%s", CurrentDir, PACKAGE_NAME_GLADE);
	g_free (CurrentDir);
	CurrentDir = NULL;
	PathShareXcfa = g_strdup_printf ("%s/basebuild/%s", DATA_DIR, PACKAGE_NAME_GLADE);
	if (libutils_test_file_exist (PathCurrentDir) == FALSE && libutils_test_file_exist (PathShareXcfa) == FALSE) {
		g_print("!---------------------------------------------------------------------------------------------------\n");
		g_print("! LE FICHIER XML: [ %s ] N'EXISTE PAS !!!\n", PathCurrentDir);
		g_print("!---------------------------------------------------------------------------------------------------\n");
		g_print("! LE FICHIER XML: [ %s ] N'EXISTE PAS !!!\n", PathShareXcfa);
		g_print("!---------------------------------------------------------------------------------------------------\n");
		
		g_free (PathCurrentDir);
		PathCurrentDir = NULL;
		g_free (PathShareXcfa);
		PathShareXcfa = NULL;
		
		// THREAD END
		gdk_threads_leave ();
		
		return (EXIT_FAILURE);
	}
	else if (libutils_test_file_exist (PathCurrentDir) == TRUE) {
		
		Path = PathCurrentDir;
	}
	else if (libutils_test_file_exist (PathShareXcfa) == TRUE) {
		
		Path = PathShareXcfa;
	}
	g_print("PATHNAME FILE : %s\n\n", Path);
	if (gtk_builder_add_from_file (GtkBuilderXcfa, Path, &p_err) <= 0) {
		g_warning ("Couldn't load builder file: %s", p_err->message);
		g_error_free (p_err);

		g_free (PathCurrentDir);
		PathCurrentDir = NULL;
		g_free (PathShareXcfa);
		PathShareXcfa = NULL;
		Path = NULL;
		
		// THREAD END
		gdk_threads_leave ();
		
		return (EXIT_FAILURE);
	}
	g_free (PathCurrentDir);
	PathCurrentDir = NULL;
	g_free (PathShareXcfa);
	PathShareXcfa = NULL;
	Path = NULL;
	gtk_builder_set_translation_domain  (GtkBuilderXcfa, gtk_set_locale ());
	
	gtk_builder_connect_signals (GtkBuilderXcfa, NULL);
	WindMain = GTK_WIDGET (GLADE_GET_OBJECT("window_main"));
	
	// REDIMENSIONNER LA FENETRE
	gtk_window_resize(GTK_WINDOW(WindMain), Config.WinWidth, Config.WinHeight);
	// PLACER LA FENETRE
	gtk_window_move(GTK_WINDOW(WindMain), Config.WinPos_X, Config.WinPos_Y);
	
	PackageVersion = g_strdup_printf ("%s %s", PACKAGE, VERSION);
	gtk_window_set_title (GTK_WINDOW (WindMain), PackageVersion);
	g_free (PackageVersion);
	PackageVersion = NULL;
	
	libutils_set_default_icone_to_win (WindMain);
	
	gtk_widget_show (WindMain);
			
	gtk_widget_set_sensitive (GTK_WIDGET(GLADE_GET_OBJECT("combobox_categories_menu")), FALSE);
	
	gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 2);
	gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 1);
	gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), 0);
	Config.BoolConfigOk = TRUE;
	gtk_notebook_set_current_page (GTK_NOTEBOOK (GLADE_GET_OBJECT("notebook_dossiers")), Config.Page);
	
	wiew_without_threads ();
	wiew_set_text_translate (_("\n\tCet onglet vous permet de consulter de facon independante \n\tles chaines de type 'translate' d'un fichier glade.\n\n\tRendez-vous dans l'onglet 'Librairies' pour la selection d'un fichier Glade\n"));
			
	gtk_main ();

	// THREAD END
	gdk_threads_leave ();
	
	exit(EXIT_SUCCESS);
}


